#ifndef __GNSCRIPTLIB_H__
#define __GNSCRIPTLIB_H__

extern "C" {
    /* initialisation */
    __declspec(dllimport) unsigned long gnscript_createcontext(const char* executablepath, const char* encodeduserinfo, char* errmsg, int errmsgsize);
    __declspec(dllimport) void gnscript_freecontext(unsigned long contextid);
    /* options */
    __declspec(dllimport) bool gnscript_option_setpermission_fileaccess_folder(unsigned long contextid, const char* folder, const char* permissions);
    __declspec(dllimport) bool gnscript_option_setpermission_fileaccess_all(unsigned long contextid, const char* permissions);
    __declspec(dllimport) bool gnscript_option_addincludepath(unsigned long contextid, const char* includefolder);

    /* working only with simple datatype string   requires license that supports strings */
    __declspec(dllimport) bool gnscript_context_settextvariable(unsigned long contextid, const char* varname, const char* value);
    __declspec(dllimport) bool gnscript_context_setintvariable(unsigned long contextid, const char* varname, __int64 value);
    __declspec(dllimport) bool gnscript_context_setdoublevariable(unsigned long contextid, const char* varname, double value);
    __declspec(dllimport) bool gnscript_context_gettextvariable(unsigned long contextid, const char* varname, const char* value, unsigned maxvaluesize);
#ifdef _WIN32
        __declspec(dllimport) bool gnscript_context_gettextvariable_bstr(unsigned long contextid, const char* varname, BSTR* value);
#endif
    __declspec(dllimport) bool gnscript_context_getintvariable(unsigned long contextid, const char* varname, __int64* pvalue);
    __declspec(dllimport) bool gnscript_context_getdoublevariable(unsigned long contextid, const char* varname, double* pvalue);
    __declspec(dllimport) bool gnscript_context_setconstant(unsigned long contextid, const char* constantname, const char* value);
    __declspec(dllimport) bool gnscript_context_definefunction_strings(unsigned long contextid, const char* functionheader, char* errmsgstring, unsigned maxerrmsgstringsize); 
    __declspec(dllimport) bool gnscript_evaluate_strings(unsigned long contextid, const char* script, char* resultstring, unsigned maxresultstringsize, char* errmsgstring, unsigned maxerrmsgstringsize); // quickly calculate an expression
    __declspec(dllimport) bool gnscript_execute_strings(unsigned long contextid, const char* scriptfile, char* resultstring, unsigned maxresultstringsize, char* errmsgstring, unsigned maxerrmsgstringsize); // run a script file

    extern "C" __declspec(dllexport) bool gnscript_callback_callfunction_strings(unsigned long contextid, const char* funcname, const char** paramvalues, unsigned paramcount, char* resultstring, unsigned maxresultstringsize, char* errmsgstring, unsigned maxerrmsgstringsize);

    __declspec(dllimport) bool gnscript_data_isnumber(const char* data);
    __declspec(dllimport) bool gnscript_data_islong(const char* data);
    __declspec(dllimport) bool gnscript_data_isdouble(const char* data);
    __declspec(dllimport) bool gnscript_data_isbool(const char* data);
    __declspec(dllimport) bool gnscript_data_getlong(const char* data, long* value);
    __declspec(dllimport) bool gnscript_data_getdouble(const char* data, double* value);
    __declspec(dllimport) bool gnscript_data_getbool(const char* data, bool* value);
#ifdef _WIN32
    __declspec(dllimport) unsigned gnscript_data_bstr_getsize(BSTR bstr, bool utf8);
    __declspec(dllimport) bool gnscript_data_bstr_gettext(BSTR bstr, char* valuestring, unsigned maxvaluestringsize, bool utf8);
#endif

    /* working with data types   requires license that additionally supports data types   strings, vectors, structures and converting numbers*/

    /* data types */
#define DATATYPE_UNDEF (0)
#define DATATYPE_STRING ('S')
#define DATATYPE_VECTOR ('[')
#define DATATYPE_STRUCTURE ('{')
#define DATATYPE_OTHER     ('p')
#define DATAID_NULL    (0)

    __declspec(dllimport) void gnscript_data_free(unsigned long contextid, unsigned long dataid);
    __declspec(dllimport) void gnscript_data_free_alluseddata(unsigned long contextid); // all used dataids
    __declspec(dllimport) unsigned long gnscript_data_duplicate(unsigned long contextid, unsigned long dataid);
    __declspec(dllimport) unsigned long gnscript_data_createvector(unsigned long contextid);
    __declspec(dllimport) unsigned long gnscript_data_createstructure(unsigned long contextid);
    __declspec(dllimport) unsigned long gnscript_data_createstring(unsigned long contextid, const char* text);
    __declspec(dllimport) unsigned long gnscript_data_createbool(unsigned long contextid, bool value);
    __declspec(dllimport) unsigned long gnscript_data_createlong(unsigned long contextid, long value);
    __declspec(dllimport) unsigned long gnscript_data_createdouble(unsigned long contextid, double value);
    __declspec(dllimport) bool gnscript_data_setreference_value(unsigned long contextid, unsigned long refdataid, unsigned long newvaluedataid);
    __declspec(dllimport) char gnscript_data_gettype(unsigned long contextid, unsigned long dataid); 
    __declspec(dllimport) bool gnscript_data_tostring(unsigned long contextid, unsigned long dataid, char* text, unsigned maxtextsize);
    __declspec(dllimport) bool gnscript_data_tobool(unsigned long contextid, unsigned long dataid, bool* value);
    __declspec(dllimport) bool gnscript_data_tolong(unsigned long contextid, unsigned long dataid, long* value);
    __declspec(dllimport) bool gnscript_data_todouble(unsigned long contextid, unsigned long dataid, double* value);
    __declspec(dllimport) bool gnscript_data_debug(unsigned long contextid, unsigned long dataid, char* debuginfo, unsigned maxdebuginfosize);
    __declspec(dllimport) unsigned gnscript_data_vector_getsize(unsigned long contextid, unsigned long vectorid);
    __declspec(dllimport) bool gnscript_data_vector_getelement(unsigned long contextid, unsigned long vectorid, unsigned index, unsigned long* elementdataid );
    __declspec(dllimport) bool gnscript_data_vector_setelement(unsigned long contextid, unsigned long vectorid, unsigned index, unsigned long elementdataid );
    __declspec(dllimport) bool gnscript_data_vector_pushelement(unsigned long contextid, unsigned long vectorid, unsigned long elementdataid );
    __declspec(dllimport) bool gnscript_data_vector_popelement(unsigned long contextid, unsigned long vectorid, unsigned long* elementdataid );
    __declspec(dllimport) bool gnscript_data_vector_clearelements(unsigned long contextid, unsigned long vectorid);
    __declspec(dllimport) unsigned gnscript_data_structure_getmembercount(unsigned long contextid, unsigned long structureid);
    __declspec(dllimport) bool gnscript_data_structure_getmembername(unsigned long contextid, unsigned long structureid, unsigned memberindex, char* membername, unsigned maxmembernamesize);
    __declspec(dllimport) bool gnscript_data_structure_getmember(unsigned long contextid, unsigned long structureid, const char* membername, unsigned long* elementdataid );
    __declspec(dllimport) bool gnscript_data_structure_setmember(unsigned long contextid, unsigned long structureid, const char* membername, unsigned long elementdataid );
    __declspec(dllimport) bool gnscript_data_structure_removemember(unsigned long contextid, unsigned long structureid, const char* membername);

    __declspec(dllimport) bool gnscript_context_setvariable(unsigned long contextid, const char* varname, unsigned long dataid);
    __declspec(dllimport) bool gnscript_context_getvariable(unsigned long contextid, const char* varname, unsigned long* dataid);
    __declspec(dllimport) bool gnscript_context_definefunction_dataids(unsigned long contextid, const char* functionheader, char* errmsgstring, unsigned maxerrmsgstringsize);
    __declspec(dllimport) bool gnscript_evaluate_dataids(unsigned long contextid, const char* script, unsigned long* resultdataid, char* errmsgstring, unsigned maxerrmsgstringsize); // quickly calculate an expression
    __declspec(dllimport) bool gnscript_execute_dataids(unsigned long contextid, const char* scriptfile, unsigned long* resultdataid, char* errmsgstring, unsigned maxerrmsgstringsize); // run a script file
    extern "C" __declspec(dllexport) bool gnscript_callback_callfunction_dataids(unsigned long contextid, const char* funcname, unsigned long* paramids, unsigned paramcount, unsigned long* resultdataid, char* errmsgstring, unsigned maxerrmsgstringsize);
    
    
    /* working with live variables  requires a license that also supports live data:   live data is data in your program that is accessed/modified by variable, structure, vector inside script, this is complicate but avoids copying data to the script context  */
    __declspec(dllimport) unsigned long gnscript_livedata_create(unsigned long contextid, void* userdataref, char datatype, const char* livedatatypename, char* errmsgstring, unsigned maxerrmsgstringsize);
    __declspec(dllimport) char gnscript_livedata_getdatatype(unsigned long contextid, void* userdataref);
    __declspec(dllimport) bool gnscript_livedata_gettypename(unsigned long contextid, void* userdataref, char* typenamestring, unsigned maxtypenamestringsize);
    __declspec(dllimport) void* gnscript_livedata_getuserdataref(unsigned long contextid, unsigned long livedataid);
    extern "C" __declspec(dllexport) bool gnscript_callback_livedata_free(unsigned long contextid, void* userdataref);    
    // DATATYPE_STRING:
    extern "C" __declspec(dllexport) bool gnscript_callback_livedata_setstring(unsigned long contextid, void* userdataref, const char* value, char* errmsgstring, unsigned maxerrmsgstringsize);    
    extern "C" __declspec(dllexport) bool gnscript_callback_livedata_getstring(unsigned long contextid, void* userdataref, char* value, unsigned maxvaluestringsize, char* errmsgstring, unsigned maxerrmsgstringsize);    

    // DATATYPE_VECTOR
    extern "C" __declspec(dllexport) bool gnscript_callback_livedata_vector_getsize(unsigned long contextid, void* userdataref, unsigned* vectorsize, char* errmsgstring, unsigned maxerrmsgstringsize);    
    extern "C" __declspec(dllexport) bool gnscript_callback_livedata_vector_getelement(unsigned long contextid, void* userdataref, int index, unsigned long* elementdataidreference, char* errmsgstring, unsigned maxerrmsgstringsize);    
    extern "C" __declspec(dllexport) bool gnscript_callback_livedata_vector_setelement(unsigned long contextid, void* userdataref, int index, unsigned long elementdataidreference, char* errmsgstring, unsigned maxerrmsgstringsize);    
    extern "C" __declspec(dllexport) bool gnscript_callback_livedata_vector_insertelement(unsigned long contextid, void* userdataref, int beforeindex, unsigned long elementdataidreference, char* errmsgstring, unsigned maxerrmsgstringsize);    
    extern "C" __declspec(dllexport) bool gnscript_callback_livedata_vector_removeelements(unsigned long contextid, void* userdataref, int fromindex, int tillindex, char* errmsgstring, unsigned maxerrmsgstringsize);    
    // DATATYPE_STRUCTURE
    extern "C" __declspec(dllexport) bool gnscript_callback_livedata_structure_getmembercount(unsigned long contextid, void* userdataref, unsigned* membercount, char* errmsgstring, unsigned maxerrmsgstringsize);    
    extern "C" __declspec(dllexport) bool gnscript_callback_livedata_structure_getmembername(unsigned long contextid, void* userdataref, unsigned memberindex, char* membername, unsigned maxmembernamestringsize, char* errmsgstring, unsigned maxerrmsgstringsize);    
    extern "C" __declspec(dllexport) bool gnscript_callback_livedata_structure_getmember(unsigned long contextid, void* userdataref, const char* membername, unsigned long* memberdataidreference, char* errmsgstring, unsigned maxerrmsgstringsize);    
    extern "C" __declspec(dllexport) bool gnscript_callback_livedata_structure_removemember(unsigned long contextid, void* userdataref, const char* membername, char* errmsgstring, unsigned maxerrmsgstringsize);
}; // extern "C"

#endif __GNSCRIPTLIB_H__
