#include <windows.h>
#include "loadgnscriptlib.h"

#ifndef USEIMPORTLIB
static HMODULE hgnscriptlibmodule = 0;
typedef unsigned long (*type_gnscript_createcontext)(const char* executablepath, const char* encodeduserinfo, char* errmsg, int errmsgsize);
typedef void (*type_gnscript_freecontext)(unsigned long contextid);
typedef bool (*type_gnscript_context_settextvariable)(unsigned long contextid, const char* varname, const char* value);
typedef bool (*type_gnscript_context_setintvariable)(unsigned long contextid, const char* varname, __int64 value);
typedef bool (*type_gnscript_context_setdoublevariable)(unsigned long contextid, const char* varname, double value);
typedef bool (*type_gnscript_context_gettextvariable)(unsigned long contextid, const char* varname, const char* value, unsigned maxvaluesize);
typedef bool (*type_gnscript_context_gettextvariable_bstr)(unsigned long contextid, const char* varname, BSTR* value);

typedef bool (*type_gnscript_context_getintvariable)(unsigned long contextid, const char* varname, __int64* pvalue);
typedef bool (*type_gnscript_context_getdoublevariable)(unsigned long contextid, const char* varname, double* pvalue);
typedef bool (*type_gnscript_context_setconstant)(unsigned long contextid, const char* constantname, const char* value);
typedef bool (*type_gnscript_context_definefunction_strings)(unsigned long contextid, const char* functionheader, char* errmsgstring, unsigned maxerrmsgstringsize); 
typedef bool (*type_gnscript_data_isnumber)(const char* data);
typedef bool (*type_gnscript_data_islong)(const char* data);
typedef bool (*type_gnscript_data_isdouble)(const char* data);
typedef bool (*type_gnscript_data_isbool)(const char* data);
typedef bool (*type_gnscript_data_getlong)(const char* data, long* value);
typedef bool (*type_gnscript_data_getdouble)(const char* data, double* value);
typedef bool (*type_gnscript_data_getbool)(const char* data, bool* value);
typedef unsigned (*type_gnscript_data_bstr_getsize)(BSTR bstr, bool utf8);
typedef bool (*type_gnscript_data_bstr_gettext)(BSTR bstr, char* valuestring, unsigned maxvaluestringsize, bool utf8);


typedef bool (*type_gnscript_option_setpermission_fileaccess_folder)(unsigned long contextid, const char* folder, const char* permissions);
typedef bool (*type_gnscript_option_setpermission_fileaccess_all)(unsigned long contextid, const char* permissions);
typedef bool (*type_gnscript_option_addincludepath)(unsigned long contextid, const char* includefolder);

typedef bool (*type_gnscript_evaluate_strings)(unsigned long contextid, const char* script, char* resultstring, unsigned maxresultstringsize, char* errmsgstring, unsigned maxerrmsgstringsize); // quickly calculate an expression
typedef bool (*type_gnscript_execute_strings)(unsigned long contextid, const char* scriptfile, char* resultstring, unsigned maxresultstringsize, char* errmsgstring, unsigned maxerrmsgstringsize); // run a script file

typedef void (*type_gnscript_data_free)(unsigned long contextid, unsigned long dataid);
typedef void (*type_gnscript_data_free_alluseddata)(unsigned long contextid); // all used dataids
typedef unsigned long (*type_gnscript_data_duplicate)(unsigned long contextid, unsigned long dataid);
typedef unsigned long (*type_gnscript_data_createvector)(unsigned long contextid);
typedef unsigned long (*type_gnscript_data_createstructure)(unsigned long contextid);
typedef unsigned long (*type_gnscript_data_createstring)(unsigned long contextid, const char* text);
typedef unsigned long (*type_gnscript_data_createbool)(unsigned long contextid, bool value);
typedef unsigned long (*type_gnscript_data_createlong)(unsigned long contextid, long value);
typedef unsigned long (*type_gnscript_data_createdouble)(unsigned long contextid, double value);
typedef bool (*type_gnscript_data_setreference_value)(unsigned long contextid, unsigned long refdataid, unsigned long newvaluedataid);
typedef char (*type_gnscript_data_gettype)(unsigned long contextid, unsigned long dataid); 
typedef bool (*type_gnscript_data_tostring)(unsigned long contextid, unsigned long dataid, char* text, unsigned maxtextsize);
typedef bool (*type_gnscript_data_tobool)(unsigned long contextid, unsigned long dataid, bool* value);
typedef bool (*type_gnscript_data_tolong)(unsigned long contextid, unsigned long dataid, long* value);
typedef bool (*type_gnscript_data_todouble)(unsigned long contextid, unsigned long dataid, double* value);
typedef bool (*type_gnscript_data_debug)(unsigned long contextid, unsigned long dataid, char* debuginfo, unsigned maxdebuginfosize);
typedef unsigned (*type_gnscript_data_vector_getsize)(unsigned long contextid, unsigned long vectorid);
typedef bool (*type_gnscript_data_vector_getelement)(unsigned long contextid, unsigned long vectorid, unsigned index, unsigned long* elementdataid );
typedef bool (*type_gnscript_data_vector_setelement)(unsigned long contextid, unsigned long vectorid, unsigned index, unsigned long elementdataid );
typedef bool (*type_gnscript_data_vector_pushelement)(unsigned long contextid, unsigned long vectorid, unsigned long elementdataid );
typedef bool (*type_gnscript_data_vector_popelement)(unsigned long contextid, unsigned long vectorid, unsigned long* elementdataid );
typedef bool (*type_gnscript_data_vector_clearelements)(unsigned long contextid, unsigned long vectorid);
typedef unsigned (*type_gnscript_data_structure_getmembercount)(unsigned long contextid, unsigned long structureid);
typedef bool (*type_gnscript_data_structure_getmembername)(unsigned long contextid, unsigned long structureid, unsigned memberindex, char* membername, unsigned maxmembernamesize);
typedef bool (*type_gnscript_data_structure_getmember)(unsigned long contextid, unsigned long structureid, const char* elementname, unsigned long* elementdataid );
typedef bool (*type_gnscript_data_structure_setmember)(unsigned long contextid, unsigned long structureid, const char* elementname, unsigned long elementdataid );
typedef bool (*type_gnscript_data_structure_removemember)(unsigned long contextid, unsigned long structureid, const char* elementname);

typedef bool (*type_gnscript_context_setvariable)(unsigned long contextid, const char* varname, unsigned long dataid);
typedef bool (*type_gnscript_context_getvariable)(unsigned long contextid, const char* varname, unsigned long* dataid);
typedef bool (*type_gnscript_context_definefunction_dataids)(unsigned long contextid, const char* functionheader, char* errmsgstring, unsigned maxerrmsgstringsize);
typedef bool (*type_gnscript_evaluate_dataids)(unsigned long contextid, const char* script, unsigned long* resultdataid, char* errmsgstring, unsigned maxerrmsgstringsize); // quickly calculate an expression
typedef bool (*type_gnscript_execute_dataids)(unsigned long contextid, const char* scriptfile, unsigned long* resultdataid, char* errmsgstring, unsigned maxerrmsgstringsize); // run a script file

typedef unsigned long (*type_gnscript_livedata_create)(unsigned long contextid, void* userdataref, char datatype, const char* livedatatypename, char* errmsgstring, unsigned maxerrmsgstringsize);
typedef char (*type_gnscript_livedata_getdatatype)(unsigned long contextid, void* userdataref);
typedef bool (*type_gnscript_livedata_gettypename)(unsigned long contextid, void* userdataref, char* typenamestring, unsigned maxtypenamestringsize);
typedef void* (*type_gnscript_livedata_getuserdataref)(unsigned long contextid, unsigned long livedataid);

type_gnscript_createcontext proc_gnscript_createcontext = 0;
type_gnscript_freecontext proc_gnscript_freecontext = 0;
type_gnscript_context_settextvariable proc_gnscript_context_settextvariable = 0;
type_gnscript_context_setintvariable proc_gnscript_context_setintvariable = 0;
type_gnscript_context_setdoublevariable proc_gnscript_context_setdoublevariable = 0;
type_gnscript_context_gettextvariable proc_gnscript_context_gettextvariable = 0;
type_gnscript_context_gettextvariable_bstr proc_gnscript_context_gettextvariable_bstr = 0;

type_gnscript_context_getintvariable proc_gnscript_context_getintvariable = 0;
type_gnscript_context_getdoublevariable proc_gnscript_context_getdoublevariable = 0;
type_gnscript_context_setconstant proc_gnscript_context_setconstant = 0;
type_gnscript_context_definefunction_strings proc_gnscript_context_definefunction_strings = 0; 
type_gnscript_data_isnumber proc_gnscript_data_isnumber = 0;
type_gnscript_data_islong proc_gnscript_data_islong = 0;
type_gnscript_data_isdouble proc_gnscript_data_isdouble = 0;
type_gnscript_data_isbool proc_gnscript_data_isbool = 0;
type_gnscript_data_getlong proc_gnscript_data_getlong = 0;
type_gnscript_data_getdouble proc_gnscript_data_getdouble = 0;
type_gnscript_data_getbool proc_gnscript_data_getbool = 0;
type_gnscript_data_bstr_getsize proc_gnscript_data_bstr_getsize = 0;
type_gnscript_data_bstr_gettext proc_gnscript_data_bstr_gettext = 0;


type_gnscript_option_setpermission_fileaccess_folder proc_gnscript_option_setpermission_fileaccess_folder = 0;
type_gnscript_option_setpermission_fileaccess_all proc_gnscript_option_setpermission_fileaccess_all = 0;
type_gnscript_option_addincludepath proc_gnscript_option_addincludepath = 0;

type_gnscript_evaluate_strings proc_gnscript_evaluate_strings = 0; // quickly calculate an expression
type_gnscript_execute_strings proc_gnscript_execute_strings = 0; // run a script file

type_gnscript_data_free proc_gnscript_data_free = 0;
type_gnscript_data_free_alluseddata proc_gnscript_data_free_alluseddata = 0; // all used dataids
type_gnscript_data_duplicate proc_gnscript_data_duplicate = 0;
type_gnscript_data_createvector proc_gnscript_data_createvector = 0;
type_gnscript_data_createstructure proc_gnscript_data_createstructure = 0;
type_gnscript_data_createstring proc_gnscript_data_createstring = 0;
type_gnscript_data_createbool proc_gnscript_data_createbool = 0;
type_gnscript_data_createlong proc_gnscript_data_createlong = 0;
type_gnscript_data_createdouble proc_gnscript_data_createdouble = 0;
type_gnscript_data_setreference_value proc_gnscript_data_setreference_value = 0;
type_gnscript_data_gettype proc_gnscript_data_gettype = 0; 
type_gnscript_data_tostring proc_gnscript_data_tostring = 0;
type_gnscript_data_tobool proc_gnscript_data_tobool = 0;
type_gnscript_data_tolong proc_gnscript_data_tolong = 0;
type_gnscript_data_todouble proc_gnscript_data_todouble = 0;
type_gnscript_data_debug proc_gnscript_data_debug = 0;
type_gnscript_data_vector_getsize proc_gnscript_data_vector_getsize = 0;
type_gnscript_data_vector_getelement proc_gnscript_data_vector_getelement = 0;
type_gnscript_data_vector_setelement proc_gnscript_data_vector_setelement = 0;
type_gnscript_data_vector_pushelement proc_gnscript_data_vector_pushelement = 0;
type_gnscript_data_vector_popelement proc_gnscript_data_vector_popelement = 0;
type_gnscript_data_vector_clearelements proc_gnscript_data_vector_clearelements = 0;
type_gnscript_data_structure_getmembercount proc_gnscript_data_structure_getmembercount = 0;
type_gnscript_data_structure_getmembername proc_gnscript_data_structure_getmembername = 0;
type_gnscript_data_structure_getmember proc_gnscript_data_structure_getmember = 0;
type_gnscript_data_structure_setmember proc_gnscript_data_structure_setmember = 0;
type_gnscript_data_structure_removemember proc_gnscript_data_structure_removemember = 0;

type_gnscript_context_setvariable proc_gnscript_context_setvariable = 0;
type_gnscript_context_getvariable proc_gnscript_context_getvariable = 0;
type_gnscript_context_definefunction_dataids proc_gnscript_context_definefunction_dataids = 0;
type_gnscript_evaluate_dataids proc_gnscript_evaluate_dataids = 0; // quickly calculate an expression
type_gnscript_execute_dataids proc_gnscript_execute_dataids = 0; // run a script file

type_gnscript_livedata_create proc_gnscript_livedata_create = 0;
type_gnscript_livedata_getdatatype proc_gnscript_livedata_getdatatype = 0;
type_gnscript_livedata_gettypename proc_gnscript_livedata_gettypename = 0;
type_gnscript_livedata_getuserdataref proc_gnscript_livedata_getuserdataref = 0;



#endif // USEIMPORTLIB

static void safestringcopy(char* buf, unsigned bufsize, const char* text)
{
    if (!buf)
        return;
    if (bufsize == 0)
        return;
    if (!text)
    {
        *buf = 0;
        return;
    }
    strcpy_s(buf, bufsize, text);
    buf[bufsize-1] = 0;
}

#define LOADFUNCTION(funcname) proc_##funcname = (type_##funcname)GetProcAddress(hgnscriptlibmodule, #funcname);\
    if (!proc_##funcname) \
    { \
    safestringcopy(errmsgstring, maxerrmsgstringsize, #funcname " not found in gnscriptlib.dll");\
        return false;\
    }

bool gnscriptlib_loadlibrary(const char* dllpath, char* errmsgstring, unsigned maxerrmsgstringsize)
{
#ifdef USEIMPORTLIB
    return true;
#else
    safestringcopy(errmsgstring, maxerrmsgstringsize, "");
    const char* path = dllpath && *dllpath ? dllpath : "gnscriptlib.dll";
    hgnscriptlibmodule = LoadLibrary(path);
    if (!hgnscriptlibmodule)
    {
        safestringcopy(errmsgstring, maxerrmsgstringsize, "gnscriptlib.dll not found");
        return false;
    }
    LOADFUNCTION(gnscript_createcontext);
    LOADFUNCTION(gnscript_freecontext);
    LOADFUNCTION(gnscript_context_settextvariable);
    LOADFUNCTION(gnscript_context_setintvariable);
    LOADFUNCTION(gnscript_context_setdoublevariable);
    LOADFUNCTION(gnscript_context_gettextvariable);
    LOADFUNCTION(gnscript_context_gettextvariable_bstr);
    LOADFUNCTION(gnscript_context_getintvariable);
    LOADFUNCTION(gnscript_context_getdoublevariable);
    LOADFUNCTION(gnscript_context_setconstant);
    LOADFUNCTION(gnscript_context_definefunction_strings);
    LOADFUNCTION(gnscript_option_setpermission_fileaccess_folder);
    LOADFUNCTION(gnscript_option_setpermission_fileaccess_all);
    LOADFUNCTION(gnscript_option_addincludepath);
    LOADFUNCTION(gnscript_evaluate_strings);
    LOADFUNCTION(gnscript_execute_strings);
    LOADFUNCTION(gnscript_data_isnumber);
    LOADFUNCTION(gnscript_data_islong);
    LOADFUNCTION(gnscript_data_isdouble);
    LOADFUNCTION(gnscript_data_isbool);
    LOADFUNCTION(gnscript_data_getlong);
    LOADFUNCTION(gnscript_data_getdouble);
    LOADFUNCTION(gnscript_data_getbool);
    LOADFUNCTION(gnscript_data_bstr_getsize);
    LOADFUNCTION(gnscript_data_bstr_gettext);

    LOADFUNCTION(gnscript_data_free);
    LOADFUNCTION(gnscript_data_free_alluseddata);
    LOADFUNCTION(gnscript_data_duplicate);
    LOADFUNCTION(gnscript_data_createvector);
    LOADFUNCTION(gnscript_data_createstructure);
    LOADFUNCTION(gnscript_data_createstring);
    LOADFUNCTION(gnscript_data_createbool);
    LOADFUNCTION(gnscript_data_createlong);
    LOADFUNCTION(gnscript_data_createdouble);
    LOADFUNCTION(gnscript_data_setreference_value);
    LOADFUNCTION(gnscript_data_gettype);
    LOADFUNCTION(gnscript_data_tostring);
    LOADFUNCTION(gnscript_data_tobool);
    LOADFUNCTION(gnscript_data_tolong);
    LOADFUNCTION(gnscript_data_todouble);
    LOADFUNCTION(gnscript_data_debug);
    LOADFUNCTION(gnscript_data_vector_getsize);
    LOADFUNCTION(gnscript_data_vector_getelement);
    LOADFUNCTION(gnscript_data_vector_setelement);
    LOADFUNCTION(gnscript_data_vector_pushelement);
    LOADFUNCTION(gnscript_data_vector_popelement);
    LOADFUNCTION(gnscript_data_vector_clearelements);
    LOADFUNCTION(gnscript_data_structure_getmembercount);
    LOADFUNCTION(gnscript_data_structure_getmembername);
    LOADFUNCTION(gnscript_data_structure_getmember);
    LOADFUNCTION(gnscript_data_structure_setmember);
    LOADFUNCTION(gnscript_data_structure_removemember);
    LOADFUNCTION(gnscript_context_setvariable);
    LOADFUNCTION(gnscript_context_getvariable);
    LOADFUNCTION(gnscript_context_definefunction_dataids);
    LOADFUNCTION(gnscript_evaluate_dataids);
    LOADFUNCTION(gnscript_execute_dataids);
    LOADFUNCTION(gnscript_livedata_create);
    LOADFUNCTION(gnscript_livedata_getdatatype);
    LOADFUNCTION(gnscript_livedata_gettypename);
    LOADFUNCTION(gnscript_livedata_getuserdataref);
    return true;
#endif // USEIMPORTLIB
}

#ifndef USEIMPORTLIB
static bool error_dllnotloaded(char* errmsgstring, unsigned maxerrmsgstringsize)
{
    safestringcopy(errmsgstring, maxerrmsgstringsize, "error: gnscriptlib.dll is not loaded");
    return false;
}
#endif
void gnscriptlib_freelibrary()
{
#ifndef USEIMPORTLIB
    if (hgnscriptlibmodule != 0)
    {
        FreeLibrary(hgnscriptlibmodule);
        hgnscriptlibmodule = 0;
    }
#endif
}


unsigned long gnscriptlib_createcontext(const char* executablepath, const char* encodeduserinfo, char* errmsg, int errmsgsize)
{
#ifdef USEIMPORTLIB
    return gnscript_createcontext(username, pw);
#else
    if (!proc_gnscript_createcontext)
        return false;
    return (*proc_gnscript_createcontext)(executablepath, encodeduserinfo, errmsg, errmsgsize);
#endif
}

void gnscriptlib_freecontext(unsigned long contextid)
{
#ifdef USEIMPORTLIB
    gnscript_freecontext(contextid);
#else
    if (!proc_gnscript_freecontext)
        return;
    (*proc_gnscript_freecontext)(contextid);
#endif
}

bool gnscriptlib_context_settextvariable(unsigned long contextid, const char* varname, const char* value)
{
#ifdef USEIMPORTLIB
    return gnscript_context_settextvariable(contextid, varname, value);
#else
    if (!proc_gnscript_context_settextvariable)
        return false;
    return (*proc_gnscript_context_settextvariable)(contextid, varname, value);
#endif
}

bool gnscriptlib_context_setintvariable(unsigned long contextid, const char* varname, __int64 value)
{
#ifdef USEIMPORTLIB
    return gnscript_context_setintvariable(contextid, varname, value);
#else
    if (!proc_gnscript_context_setintvariable)
        return false;
    return (*proc_gnscript_context_setintvariable)(contextid, varname, value);
#endif
}

bool gnscriptlib_context_setdoublevariable(unsigned long contextid, const char* varname, double value)
{
#ifdef USEIMPORTLIB
    return gnscript_context_setdoublevariable(contextid, varname, value);
#else
    if (!proc_gnscript_context_setdoublevariable)
        return false;
    return (*proc_gnscript_context_setdoublevariable)(contextid, varname, value);
#endif
}

bool gnscriptlib_context_gettextvariable(unsigned long contextid, const char* varname, const char* value, unsigned maxvaluesize)
{
#ifdef USEIMPORTLIB
    return gnscript_context_gettextvariable(contextid, varname, value, maxvaluesize);
#else
    if (!proc_gnscript_context_gettextvariable)
        return false;
    return (*proc_gnscript_context_gettextvariable)(contextid, varname, value, maxvaluesize);
#endif
}

bool gnscriptlib_context_gettextvariable_bstr(unsigned long contextid, const char* varname, BSTR* value)
{
#ifdef USEIMPORTLIB
    return gnscript_context_gettextvariable_bstr(contextid, varname, value);
#else
    if (!proc_gnscript_context_gettextvariable_bstr)
        return false;
    return (*proc_gnscript_context_gettextvariable_bstr)(contextid, varname, value);
#endif
}

bool gnscriptlib_context_getintvariable(unsigned long contextid, const char* varname, __int64* pvalue)
{
#ifdef USEIMPORTLIB
    return gnscript_context_getintvariable(contextid, varname, pvalue);
#else
    if (!proc_gnscript_context_getintvariable)
        return false;
    return (*proc_gnscript_context_getintvariable)(contextid, varname, pvalue);
#endif
}

bool gnscriptlib_context_getdoublevariable(unsigned long contextid, const char* varname, double* pvalue)
{
#ifdef USEIMPORTLIB
    return gnscript_context_getdoublevariable(contextid, varname, pvalue);
#else
    if (!proc_gnscript_context_getdoublevariable)
        return false;
    return (*proc_gnscript_context_getdoublevariable)(contextid, varname, pvalue);
#endif
}

bool gnscriptlib_context_setconstant(unsigned long contextid, const char* constantname, const char* value)
{
#ifdef USEIMPORTLIB
    return gnscript_context_setconstant(contextid, constantname, value);
#else
    if (!proc_gnscript_context_setconstant)
        return false;
    return (*proc_gnscript_context_setconstant)(contextid, constantname, value);
#endif
}

bool gnscriptlib_context_definefunction_strings(unsigned long contextid, const char* functionheader, char* errmsgstring, unsigned maxerrmsgstringsize)
{
#ifdef USEIMPORTLIB
    return gnscript_context_definefunction_strings(contextid, functionheader, errmsgstring, maxerrmsgstringsize);
#else
    if (!proc_gnscript_context_definefunction_strings)
        return false;
    return (*proc_gnscript_context_definefunction_strings)(contextid, functionheader, errmsgstring, maxerrmsgstringsize);
#endif
}

bool gnscriptlib_option_setpermission_fileaccess_folder(unsigned long contextid, const char* folder, const char* permissions)
{
#ifdef USEIMPORTLIB
    return gnscript_option_setpermission_fileaccess_folder(contextid, folder, permissions);
#else
    if (!proc_gnscript_option_setpermission_fileaccess_folder)
        return false;
    return (*proc_gnscript_option_setpermission_fileaccess_folder)(contextid, folder, permissions);
#endif
}

bool gnscriptlib_option_setpermission_fileaccess_all(unsigned long contextid, const char* permissions)
{
#ifdef USEIMPORTLIB
    return gnscript_option_setpermission_fileaccess_all(contextid, permissions);
#else
    if (!proc_gnscript_option_setpermission_fileaccess_all)
        return false;
    return (*proc_gnscript_option_setpermission_fileaccess_all)(contextid, permissions);
#endif
}

bool gnscriptlib_option_addincludepath(unsigned long contextid, const char* includefolder)
{
#ifdef USEIMPORTLIB
    return gnscript_option_addincludepath(contextid, includefolder);
#else
    if (!proc_gnscript_option_addincludepath)
        return false;
    return (*proc_gnscript_option_addincludepath)(contextid, includefolder);
#endif
}

bool gnscriptlib_evaluate_strings(unsigned long contextid, const char* script, char* resultstring, unsigned maxresultstringsize, char* errmsgstring, unsigned maxerrmsgstringsize) // quickly calculate an expression
{
#ifdef USEIMPORTLIB
    return gnscript_evaluate_strings(contextid, script, resultstring, maxresultstringsize, errmsgstring, maxerrmsgstringsize);
#else
    if (!proc_gnscript_evaluate_strings)
        return false;
    return (*proc_gnscript_evaluate_strings)(contextid, script, resultstring, maxresultstringsize, errmsgstring, maxerrmsgstringsize);
#endif
}

bool gnscriptlib_execute_strings(unsigned long contextid, const char* scriptfile, char* resultstring, unsigned maxresultstringsize, char* errmsgstring, unsigned maxerrmsgstringsize) // run a script file
{
#ifdef USEIMPORTLIB
    return gnscript_execute_strings(contextid, scriptfile, resultstring, maxresultstringsize, errmsgstring, maxerrmsgstringsize);
#else
    if (!proc_gnscript_execute_strings)
        return false;
    return (*proc_gnscript_execute_strings)(contextid, scriptfile, resultstring, maxresultstringsize, errmsgstring, maxerrmsgstringsize);
#endif
}

bool gnscriptlib_data_isnumber(const char* data)
{
#ifdef USEIMPORTLIB
    return gnscript_data_isnumber(data);
#else
    if (!proc_gnscript_data_isnumber)
        return false;
    return (*proc_gnscript_data_isnumber)(data);
#endif
}
bool gnscriptlib_data_islong(const char* data)
{
#ifdef USEIMPORTLIB
    return gnscript_data_islong(data);
#else
    if (!proc_gnscript_data_islong)
        return false;
    return (*proc_gnscript_data_islong)(data);
#endif
}
bool gnscriptlib_data_isdouble(const char* data)
{
#ifdef USEIMPORTLIB
    return gnscript_data_isdouble(data);
#else
    if (!proc_gnscript_data_isdouble)
        return false;
    return (*proc_gnscript_data_isdouble)(data);
#endif
}

bool gnscriptlib_data_isbool(const char* data)
{
#ifdef USEIMPORTLIB
    return gnscript_data_isbool(data);
#else
    if (!proc_gnscript_data_isbool)
        return false;
    return (*proc_gnscript_data_isbool)(data);
#endif
}

bool gnscriptlib_data_getlong(const char* data, long* value)
{
#ifdef USEIMPORTLIB
    return gnscript_data_getlong(data,value);
#else
    if (!proc_gnscript_data_getlong)
        return false;
    return (*proc_gnscript_data_getlong)(data, value);
#endif
}
bool gnscriptlib_data_getdouble(const char* data, double* value)
{
#ifdef USEIMPORTLIB
    return gnscript_data_getdouble(data,value);
#else
    if (!proc_gnscript_data_getdouble)
        return false;
    return (*proc_gnscript_data_getdouble)(data, value);
#endif
}
bool gnscriptlib_data_getbool(const char* data, bool* value)
{
#ifdef USEIMPORTLIB
    return gnscript_data_getbool(data,value);
#else
    if (!proc_gnscript_data_getbool)
        return false;
    return (*proc_gnscript_data_getbool)(data, value);
#endif
}

unsigned gnscriptlib_data_bstr_getsize(BSTR bstr, bool utf8)
{
#ifdef USEIMPORTLIB
    return gnscript_data_bstr_getsize(bstr,utf8);
#else
    if (!proc_gnscript_data_bstr_getsize)
        return false;
    return (*proc_gnscript_data_bstr_getsize)(bstr, utf8);
#endif
}

bool gnscriptlib_data_bstr_gettext(BSTR bstr, char* valuestring, unsigned maxvaluestringsize, bool utf8)
{
#ifdef USEIMPORTLIB
    return gnscript_data_bstr_gettext(bstr,valuestring, maxvaluestringsize, utf8);
#else
    if (!proc_gnscript_data_bstr_gettext)
        return false;
    return (*proc_gnscript_data_bstr_gettext)(bstr, valuestring, maxvaluestringsize, utf8);
#endif
}

void gnscriptlib_data_free(unsigned long contextid, unsigned long dataid)
{
#ifdef USEIMPORTLIB
    return gnscript_data_free(contextid, dataid);
#else
    if (!proc_gnscript_data_free)
        return;
    return (*proc_gnscript_data_free)(contextid, dataid);
#endif
}

void gnscriptlib_data_free_alluseddata(unsigned long contextid) // all used dataids
{
#ifdef USEIMPORTLIB
    return gnscript_data_free_alluseddata(contextid);
#else
    if (!proc_gnscript_data_free_alluseddata)
        return;
    return (*proc_gnscript_data_free_alluseddata)(contextid);
#endif
}

unsigned long gnscriptlib_data_duplicate(unsigned long contextid, unsigned long dataid)
{
#ifdef USEIMPORTLIB
    return gnscript_data_duplicate(contextid, dataid);
#else
    if (!proc_gnscript_data_duplicate)
        return 0;
    return (*proc_gnscript_data_duplicate)(contextid, dataid);
#endif
}

unsigned long gnscriptlib_data_createvector(unsigned long contextid)
{
#ifdef USEIMPORTLIB
    return gnscript_data_createvector(contextid);
#else
    if (!proc_gnscript_data_createvector)
        return 0;
    return (*proc_gnscript_data_createvector)(contextid);
#endif
}

unsigned long gnscriptlib_data_createstructure(unsigned long contextid)
{
#ifdef USEIMPORTLIB
    return gnscript_data_createstructure(contextid);
#else
    if (!proc_gnscript_data_createstructure)
        return 0;
    return (*proc_gnscript_data_createstructure)(contextid);
#endif
}

unsigned long gnscriptlib_data_createstring(unsigned long contextid, const char* text)
{
#ifdef USEIMPORTLIB
    return gnscript_data_createstring(contextid, text);
#else
    if (!proc_gnscript_data_createstring)
        return 0;
    return (*proc_gnscript_data_createstring)(contextid, text);
#endif
}

unsigned long gnscriptlib_data_createbool(unsigned long contextid, bool value)
{
#ifdef USEIMPORTLIB
    return gnscript_data_createbool(contextid, value);
#else
    if (!proc_gnscript_data_createbool)
        return 0;
    return (*proc_gnscript_data_createbool)(contextid, value);
#endif
}

unsigned long gnscriptlib_data_createlong(unsigned long contextid, long value)
{
#ifdef USEIMPORTLIB
    return gnscript_data_createlong(contextid, value);
#else
    if (!proc_gnscript_data_createlong)
        return 0;
    return (*proc_gnscript_data_createlong)(contextid, value);
#endif
}

unsigned long gnscriptlib_data_createdouble(unsigned long contextid, double value)
{
#ifdef USEIMPORTLIB
    return gnscript_data_createdouble(contextid, value);
#else
    if (!proc_gnscript_data_createdouble)
        return 0;
    return (*proc_gnscript_data_createdouble)(contextid, value);
#endif
}

bool gnscriptlib_data_setreference_value(unsigned long contextid, unsigned long refdataid, unsigned long newvaluedataid)
{
#ifdef USEIMPORTLIB
    return gnscript_data_setreference_value(contextid, refdataid, newvaluedataid);
#else
    if (!proc_gnscript_data_setreference_value)
        return 0;
    return (*proc_gnscript_data_setreference_value)(contextid, refdataid, newvaluedataid);
#endif
}

char gnscriptlib_data_gettype(unsigned long contextid, unsigned long dataid) 
{
#ifdef USEIMPORTLIB
    return gnscript_data_gettype(contextid, dataid);
#else
    if (!proc_gnscript_data_gettype)
        return 0;
    return (*proc_gnscript_data_gettype)(contextid, dataid);
#endif
}

bool gnscriptlib_data_tostring(unsigned long contextid, unsigned long dataid, char* text, unsigned maxtextsize)
{
#ifdef USEIMPORTLIB
    return gnscript_data_tostring(contextid, dataid, text, maxtextsize);
#else
    if (!proc_gnscript_data_tostring)
        return false;
    return (*proc_gnscript_data_tostring)(contextid, dataid, text, maxtextsize);
#endif
}

bool gnscriptlib_data_tobool(unsigned long contextid, unsigned long dataid, bool* value)
{
#ifdef USEIMPORTLIB
    return gnscript_data_tobool(contextid, dataid, value);
#else
    if (!proc_gnscript_data_tobool)
        return false;
    return (*proc_gnscript_data_tobool)(contextid, dataid, value);
#endif
}

bool gnscriptlib_data_tolong(unsigned long contextid, unsigned long dataid, long* value)
{
#ifdef USEIMPORTLIB
    return gnscript_data_tolong(contextid, dataid, value);
#else
    if (!proc_gnscript_data_tolong)
        return false;
    return (*proc_gnscript_data_tolong)(contextid, dataid, value);
#endif
}

bool gnscriptlib_data_todouble(unsigned long contextid, unsigned long dataid, double* value)
{
#ifdef USEIMPORTLIB
    return gnscript_data_todouble(contextid, dataid, value);
#else
    if (!proc_gnscript_data_todouble)
        return false;
    return (*proc_gnscript_data_todouble)(contextid, dataid, value);
#endif
}

bool gnscriptlib_data_debug(unsigned long contextid, unsigned long dataid, char* debuginfo, unsigned maxdebuginfosize)
{
#ifdef USEIMPORTLIB
    return gnscript_data_debug(contextid, dataid, debuginfo, maxdebuginfosize);
#else
    if (!proc_gnscript_data_debug)
        return false;
    return (*proc_gnscript_data_debug)(contextid, dataid, debuginfo, maxdebuginfosize);
#endif
}

unsigned gnscriptlib_data_vector_getsize(unsigned long contextid, unsigned long vectorid)
{
#ifdef USEIMPORTLIB
    return gnscript_data_vector_getsize(contextid, vectorid);
#else
    if (!proc_gnscript_data_vector_getsize)
        return 0;
    return (*proc_gnscript_data_vector_getsize)(contextid, vectorid);
#endif
}

bool gnscriptlib_data_vector_getelement(unsigned long contextid, unsigned long vectorid, unsigned index, unsigned long* elementdataid )
{
#ifdef USEIMPORTLIB
    return gnscript_data_vector_getelement(contextid, vectorid, index, elementdataid);
#else
    if (!proc_gnscript_data_vector_getelement)
        return false;
    return (*proc_gnscript_data_vector_getelement)(contextid, vectorid, index, elementdataid);
#endif
}

bool gnscriptlib_data_vector_setelement(unsigned long contextid, unsigned long vectorid, unsigned index, unsigned long elementdataid )
{
#ifdef USEIMPORTLIB
    return gnscript_data_vector_setelement(contextid, vectorid, index, elementdataid);
#else
    if (!proc_gnscript_data_vector_setelement)
        return false;
    return (*proc_gnscript_data_vector_setelement)(contextid, vectorid, index, elementdataid);
#endif
}

bool gnscriptlib_data_vector_pushelement(unsigned long contextid, unsigned long vectorid, unsigned long elementdataid )
{
#ifdef USEIMPORTLIB
    return gnscript_data_vector_pushelement(contextid, vectorid, elementdataid);
#else
    if (!proc_gnscript_data_vector_pushelement)
        return false;
    return (*proc_gnscript_data_vector_pushelement)(contextid, vectorid, elementdataid);
#endif
}

bool gnscriptlib_data_vector_popelement(unsigned long contextid, unsigned long vectorid, unsigned long* elementdataid )
{
#ifdef USEIMPORTLIB
    return gnscript_data_vector_popelement(contextid, vectorid, elementdataid);
#else
    if (!proc_gnscript_data_vector_popelement)
        return false;
    return (*proc_gnscript_data_vector_popelement)(contextid, vectorid, elementdataid);
#endif
}

bool gnscriptlib_data_vector_clearelements(unsigned long contextid, unsigned long vectorid)
{
#ifdef USEIMPORTLIB
    return gnscript_data_vector_clearelements(contextid, vectorid);
#else
    if (!proc_gnscript_data_vector_clearelements)
        return false;
    return (*proc_gnscript_data_vector_clearelements)(contextid, vectorid);
#endif
}

unsigned gnscriptlib_data_structure_getmembercount(unsigned long contextid, unsigned long structureid)
{
#ifdef USEIMPORTLIB
    return gnscript_data_structure_getmembercount(contextid, structureid);
#else
    if (!proc_gnscript_data_structure_getmembercount)
        return 0;
    return (*proc_gnscript_data_structure_getmembercount)(contextid, structureid);
#endif
}

bool gnscriptlib_data_structure_getmembername(unsigned long contextid, unsigned long structureid, unsigned memberindex, char* membername, unsigned maxmembernamesize)
{
#ifdef USEIMPORTLIB
    return gnscript_data_structure_getmembername(contextid, structureid, memberindex, membername, maxmembernamesize);
#else
    if (!proc_gnscript_data_structure_getmembername)
        return false;
    return (*proc_gnscript_data_structure_getmembername)(contextid, structureid, memberindex, membername, maxmembernamesize);
#endif
}

bool gnscriptlib_data_structure_getmember(unsigned long contextid, unsigned long structureid, const char* membername, unsigned long* elementdataid )
{
#ifdef USEIMPORTLIB
    return gnscript_data_structure_getmember(contextid, structureid, membername, elementdataid);
#else
    if (!proc_gnscript_data_structure_getmember)
        return false;
    return (*proc_gnscript_data_structure_getmember)(contextid, structureid, membername, elementdataid);
#endif
}

bool gnscriptlib_data_structure_setmember(unsigned long contextid, unsigned long structureid, const char* membername, unsigned long elementdataid )
{
#ifdef USEIMPORTLIB
    return gnscript_data_structure_setmember(contextid, structureid, membername, elementdataid);
#else
    if (!proc_gnscript_data_structure_setmember)
        return false;
    return (*proc_gnscript_data_structure_setmember)(contextid, structureid, membername, elementdataid);
#endif
}

bool gnscriptlib_data_structure_removemember(unsigned long contextid, unsigned long structureid, const char* membername)
{
#ifdef USEIMPORTLIB
    return gnscript_data_structure_removemember(contextid, structureid, membername);
#else
    if (!proc_gnscript_data_structure_removemember)
        return false;
    return (*proc_gnscript_data_structure_removemember)(contextid, structureid, membername);
#endif
}


bool gnscriptlib_context_setvariable(unsigned long contextid, const char* varname, unsigned long dataid)
{
#ifdef USEIMPORTLIB
    return gnscript_context_setvariable(contextid, varname, dataid);
#else
    if (!proc_gnscript_context_setvariable)
        return false;
    return (*proc_gnscript_context_setvariable)(contextid, varname, dataid);
#endif
}

bool gnscriptlib_context_getvariable(unsigned long contextid, const char* varname, unsigned long* dataid)
{
#ifdef USEIMPORTLIB
    return gnscript_context_getvariable(contextid, varname, dataid);
#else
    if (!proc_gnscript_context_getvariable)
        return false;
    return (*proc_gnscript_context_getvariable)(contextid, varname, dataid);
#endif
}

bool gnscriptlib_context_definefunction_dataids(unsigned long contextid, const char* functionheader, char* errmsgstring, unsigned maxerrmsgstringsize)
{
#ifdef USEIMPORTLIB
    return gnscript_context_definefunction_dataids(contextid, functionheader, errmsgstring, maxerrmsgstringsize);
#else
    if (!proc_gnscript_context_definefunction_dataids)
        return false;
    return (*proc_gnscript_context_definefunction_dataids)(contextid, functionheader, errmsgstring, maxerrmsgstringsize);
#endif
}


bool gnscriptlib_evaluate_dataids(unsigned long contextid, const char* script, unsigned long* resultdataid, char* errmsgstring, unsigned maxerrmsgstringsize) // quickly calculate an expression
{
#ifdef USEIMPORTLIB
    return gnscript_evaluate_dataids(contextid, script, resultdataid, errmsgstring, maxerrmsgstringsize);
#else
    if (!proc_gnscript_evaluate_dataids)
        return false;
    return (*proc_gnscript_evaluate_dataids)(contextid, script, resultdataid, errmsgstring, maxerrmsgstringsize);
#endif
}

bool gnscriptlib_execute_dataids(unsigned long contextid, const char* scriptfile, unsigned long* resultdataid, char* errmsgstring, unsigned maxerrmsgstringsize) // run a script file
{
#ifdef USEIMPORTLIB
    return gnscript_execute_dataids(contextid, scriptfile, resultdataid, errmsgstring, maxerrmsgstringsize);
#else
    if (!proc_gnscript_execute_dataids)
        return false;
    return (*proc_gnscript_execute_dataids)(contextid, scriptfile, resultdataid, errmsgstring, maxerrmsgstringsize);
#endif
}


unsigned long gnscriptlib_livedata_create(unsigned long contextid, void* userdataref, char datatype, const char* livedatatypename, char* errmsgstring, unsigned maxerrmsgstringsize)
{
#ifdef USEIMPORTLIB
    return gnscript_livedata_create(contextid, userdataref, datatype, livedatatypename, errmsgstring, maxerrmsgstringsize);
#else
    if (!proc_gnscript_livedata_create)
        return false;
    return (*proc_gnscript_livedata_create)(contextid, userdataref, datatype, livedatatypename, errmsgstring, maxerrmsgstringsize);
#endif
}

char gnscriptlib_livedata_getdatatype(unsigned long contextid, void* userdataref)
{
#ifdef USEIMPORTLIB
    return gnscript_livedata_getdatatype(contextid, userdataref);
#else
    if (!proc_gnscript_livedata_getdatatype)
        return false;
    return (*proc_gnscript_livedata_getdatatype)(contextid, userdataref);
#endif
}

bool gnscriptlib_livedata_gettypename(unsigned long contextid, void* userdataref, char* typenamestring, unsigned maxtypenamestringsize)
{
#ifdef USEIMPORTLIB
    return gnscript_livedata_gettypename(contextid, userdataref, typenamestring, maxtypenamestringsize);
#else
    if (!proc_gnscript_livedata_gettypename)
        return false;
    return (*proc_gnscript_livedata_gettypename)(contextid, userdataref, typenamestring, maxtypenamestringsize);
#endif
}

void* gnscriptlib_livedata_getuserdataref(unsigned long contextid, unsigned long livedataid)
{
#ifdef USEIMPORTLIB
    return gnscript_livedata_getuserdataref(contextid, livedataid);
#else
    if (!proc_gnscript_livedata_getuserdataref)
        return false;
    return (*proc_gnscript_livedata_getuserdataref)(contextid, livedataid);
#endif
}

