#ifndef __LOADGNSCRIPTLIB_H__
#define __LOADGNSCRIPTLIB_H__

#include "gnscriptlib.h"

#ifdef USEIMPORTLIB
#else
// loadlibrary dynamically
#endif  

bool gnscriptlib_loadlibrary(const char* dllpath, char* errmsgstring, unsigned maxerrmsgstringsize);
void gnscriptlib_freelibrary();

unsigned long gnscriptlib_createcontext(const char* executablepath, const char* encodeduserinfo, char* errmsg, int errmsgsize);
void gnscriptlib_freecontext(unsigned long contextid);

bool gnscriptlib_context_settextvariable(unsigned long contextid, const char* varname, const char* value);
bool gnscriptlib_context_setintvariable(unsigned long contextid, const char* varname, __int64 value);
bool gnscriptlib_context_setdoublevariable(unsigned long contextid, const char* varname, double value);
bool gnscriptlib_context_settextvariable(unsigned long contextid, const char* varname, const char* value);
bool gnscriptlib_context_gettextvariable(unsigned long contextid, const char* varname, const char* value, unsigned maxvaluesize);
bool gnscriptlib_context_gettextvariable_bstr(unsigned long contextid, const char* varname, BSTR* value);
bool gnscriptlib_context_getintvariable(unsigned long contextid, const char* varname, __int64* pvalue);
bool gnscriptlib_context_getdoublevariable(unsigned long contextid, const char* varname, double* pvalue);
bool gnscriptlib_context_setconstant(unsigned long contextid, const char* constantname, const char* value);
bool gnscriptlib_context_definefunction_strings(unsigned long contextid, const char* functionheader, char* errmsgstring, unsigned maxerrmsgstringsize); 

bool gnscriptlib_option_setpermission_fileaccess_folder(unsigned long contextid, const char* folder, const char* permissions);
bool gnscriptlib_option_setpermission_fileaccess_all(unsigned long contextid, const char* permissions);
bool gnscriptlib_option_addincludepath(unsigned long contextid, const char* includefolder);

bool gnscriptlib_evaluate_strings(unsigned long contextid, const char* script, char* resultstring, unsigned maxresultstringsize, char* errmsgstring, unsigned maxerrmsgstringsize); // quickly calculate an expression
bool gnscriptlib_execute_strings(unsigned long contextid, const char* scriptfile, char* resultstring, unsigned maxresultstringsize, char* errmsgstring, unsigned maxerrmsgstringsize); // run a script file
bool gnscriptlib_data_isnumber(const char* data);
bool gnscriptlib_data_islong(const char* data);
bool gnscriptlib_data_isdouble(const char* data);
bool gnscriptlib_data_isbool(const char* data);
bool gnscriptlib_data_getlong(const char* data, long* value);
bool gnscriptlib_data_getdouble(const char* data, double* value);
bool gnscriptlib_data_getbool(const char* data, bool* value);
unsigned gnscriptlib_data_bstr_getsize(BSTR bstr, bool utf8);
bool gnscriptlib_data_bstr_gettext(BSTR bstr, char* valuestring, unsigned maxvaluestringsize, bool utf8);

/*
 * 
 * datatypes functions (requires license with feature datatypes)
 *
 */
void gnscriptlib_data_free(unsigned long contextid, unsigned long dataid);
void gnscriptlib_data_free_alluseddata(unsigned long contextid); // all used dataids
unsigned long gnscriptlib_data_duplicate(unsigned long contextid, unsigned long dataid);
unsigned long gnscriptlib_data_createvector(unsigned long contextid);
unsigned long gnscriptlib_data_createstructure(unsigned long contextid);
unsigned long gnscriptlib_data_createstring(unsigned long contextid, const char* text);
unsigned long gnscriptlib_data_createbool(unsigned long contextid, bool value);
unsigned long gnscriptlib_data_createlong(unsigned long contextid, long value);
unsigned long gnscriptlib_data_createdouble(unsigned long contextid, double value);
bool gnscriptlib_data_setreference_value(unsigned long contextid, unsigned long refdataid, unsigned long newvaluedataid);
char gnscriptlib_data_gettype(unsigned long contextid, unsigned long dataid); 
bool gnscriptlib_data_tostring(unsigned long contextid, unsigned long dataid, char* text, unsigned maxtextsize);
bool gnscriptlib_data_tobool(unsigned long contextid, unsigned long dataid, bool* value);
bool gnscriptlib_data_tolong(unsigned long contextid, unsigned long dataid, long* value);
bool gnscriptlib_data_todouble(unsigned long contextid, unsigned long dataid, double* value);
bool gnscriptlib_data_debug(unsigned long contextid, unsigned long dataid, char* debuginfo, unsigned maxdebuginfosize);
unsigned gnscriptlib_data_vector_getsize(unsigned long contextid, unsigned long vectorid);
bool gnscriptlib_data_vector_getelement(unsigned long contextid, unsigned long vectorid, unsigned index, unsigned long* elementdataid );
bool gnscriptlib_data_vector_setelement(unsigned long contextid, unsigned long vectorid, unsigned index, unsigned long elementdataid );
bool gnscriptlib_data_vector_pushelement(unsigned long contextid, unsigned long vectorid, unsigned long elementdataid );
bool gnscriptlib_data_vector_popelement(unsigned long contextid, unsigned long vectorid, unsigned long* elementdataid );
bool gnscriptlib_data_vector_clearelements(unsigned long contextid, unsigned long vectorid);
unsigned gnscriptlib_data_structure_getmembercount(unsigned long contextid, unsigned long structureid);
bool gnscriptlib_data_structure_getmembername(unsigned long contextid, unsigned long structureid, unsigned memberindex, char* membername, unsigned maxmembernamesize);
bool gnscriptlib_data_structure_getmember(unsigned long contextid, unsigned long structureid, const char* membername, unsigned long* elementdataid );
bool gnscriptlib_data_structure_setmember(unsigned long contextid, unsigned long structureid, const char* membername, unsigned long elementdataid );
bool gnscriptlib_data_structure_removemember(unsigned long contextid, unsigned long structureid, const char* membername);

bool gnscriptlib_context_setvariable(unsigned long contextid, const char* varname, unsigned long dataid);
bool gnscriptlib_context_getvariable(unsigned long contextid, const char* varname, unsigned long* dataid);
bool gnscriptlib_context_definefunction_dataids(unsigned long contextid, const char* functionheader, char* errmsgstring, unsigned maxerrmsgstringsize);
bool gnscriptlib_evaluate_dataids(unsigned long contextid, const char* script, unsigned long* resultdataid, char* errmsgstring, unsigned maxerrmsgstringsize); // quickly calculate an expression
bool gnscriptlib_execute_dataids(unsigned long contextid, const char* scriptfile, unsigned long* resultdataid, char* errmsgstring, unsigned maxerrmsgstringsize); // run a script file

/*
 * 
 * livedata functions (requires license with feature livedata)
 *
 */

unsigned long gnscriptlib_livedata_create(unsigned long contextid, void* userdataref, char datatype, const char* livedatatypename, char* errmsgstring, unsigned maxerrmsgstringsize);
char gnscriptlib_livedata_getdatatype(unsigned long contextid, void* userdataref);
bool gnscriptlib_livedata_gettypename(unsigned long contextid, void* userdataref, char* typenamestring, unsigned maxtypenamestringsize);
void* gnscriptlib_livedata_getuserdataref(unsigned long contextid, unsigned long livedataid);


#endif // __LOADGNSCRIPTLIB_H__