// testgnscriptlib.cpp : Definiert den Einstiegspunkt fr die Konsolenanwendung.
//
#include <iostream>
#include <Windows.h>
#include "gnscriptlib.h"
#include "loadgnscriptlib.h" // alternative  link gnscriptlib.lib and then use the functions of gnscriptlib.h directly, when using loadgnscriptlib.h then use the functions of this file
#ifdef GNUNITTEST
#include "gnscriptlib_unittests.h"
#endif
#include "userlicensedata.h"


extern "C" __declspec(dllexport) bool gnscript_callback_callfunction_strings(unsigned long contextid, const char* funcname, const char** paramvalues, unsigned paramcount, char* resultstring, unsigned maxresultstringsize, char* errmsgstring, unsigned maxerrmsgstringsize)
{
    resultstring[0] = 0;
    errmsgstring[0] = 0;
    if (!funcname)
        return false;
    if (strcmp(funcname, "calccirclearea") == 0 && paramcount == 1)
    {
        char pi_buf[20];
        char errmsg_buf[100];
        gnscriptlib_evaluate_strings(contextid, "mypi", pi_buf, sizeof(pi_buf), errmsg_buf, sizeof errmsg_buf);
        double pi;
        gnscriptlib_data_getdouble(pi_buf, &pi);
        double value;
        gnscriptlib_data_getdouble(paramvalues[0], &value);
        double area =  value * value * pi;
        sprintf_s(resultstring, maxresultstringsize, "%3.5lf", area);
        return true;
    }
#ifdef GNUNITTEST
    return gnscriptlib_unittests::gnscript_callback_callfunction_strings(contextid, funcname, paramvalues, paramcount, resultstring, maxresultstringsize, errmsgstring, maxerrmsgstringsize);
#endif
    strcpy_s(errmsgstring, maxerrmsgstringsize, "error: function not implemented ");
    strcpy_s(errmsgstring + strlen(errmsgstring), maxerrmsgstringsize-strlen(errmsgstring), funcname);
    return false;
}


extern "C" __declspec(dllexport) bool gnscript_callback_callfunction_dataids(unsigned long contextid, const char* funcname, unsigned long* paramids, unsigned paramcount, unsigned long* resultdataid, char* errmsgstring, unsigned maxerrmsgstringsize)
{
    if (!resultdataid)
    {
        strcpy_s(errmsgstring, sizeof maxerrmsgstringsize, "error: resultdataid must not be null");
        return false;
    }
    *resultdataid = 0;
    errmsgstring[0] = 0;
    if (!funcname)
        return false;
    if (strcmp(funcname, "calccirclearea2") == 0 && paramcount == 2)
    {
        char pi_buf[20];
        char errmsg_buf[100];
        gnscriptlib_evaluate_strings(contextid, "mypi", pi_buf, sizeof(pi_buf), errmsg_buf, sizeof errmsg_buf);
        double pi;
        gnscriptlib_data_getdouble(pi_buf, &pi);
        double value;
        if (!gnscriptlib_data_todouble(contextid, paramids[0], &value))
        {
            strcpy_s(errmsgstring, sizeof maxerrmsgstringsize, "error: calccirclearea2 parameter radius expects a number");
            return false;
        }

        double area =  value * value * pi;
        if (paramids[1] == 0)
        {
            strcpy_s(errmsgstring, sizeof maxerrmsgstringsize, "error: calccirclearea2 parameter area must be a reference to data");
            return false;
        }
        char evalbuf[100];
        strcpy_s(evalbuf, sizeof(evalbuf), "resultarea = ");
        sprintf_s(evalbuf + strlen(evalbuf), sizeof(evalbuf)-strlen(evalbuf), "%3.5lf", area);
        gnscriptlib_evaluate_strings(contextid, evalbuf, 0, 0, errmsgstring, sizeof errmsgstring);
//        gnscriptlib_data_setreference_value(contextid, paramids[1], gnscriptlib_data_createdouble(contextid, area));
        gnscriptlib_data_setreference_value(contextid, paramids[0], gnscriptlib_data_createlong(contextid, 13));

        *resultdataid = gnscriptlib_data_createbool(contextid, true);
        return true;
    }
#ifdef GNUNITTEST
    return gnscriptlib_unittests::gnscript_callback_callfunction_dataids(contextid, funcname, paramids, paramcount, resultdataid, errmsgstring, maxerrmsgstringsize);
#endif
    strcpy_s(errmsgstring, maxerrmsgstringsize, "error: function not implemented ");
    strcpy_s(errmsgstring + strlen(errmsgstring), maxerrmsgstringsize-strlen(errmsgstring), funcname);
    return false;
}


int main(int argc, char* argv[])
{
#ifdef GNUNITTEST
    gnscriptlib_unittests::gnscriptlib_addunittests();
#endif

    char errmsgstring[256];
    if (!gnscriptlib_loadlibrary(0, errmsgstring, sizeof errmsgstring))
    {
        std::cerr << errmsgstring << "\n";
        return 1;
    }


    char myexepath[256];
    GetModuleFileName(NULL, myexepath, sizeof myexepath);
    unsigned long contextid = gnscriptlib_createcontext(myexepath, USERLICENSEDATA, errmsgstring, sizeof(errmsgstring));
    if (!contextid)
    {
        std::cerr << errmsgstring << '\n';
        return 1;
    }

    // if you want to write a file in script then you need to enable permissions for all folders or certain folders
    gnscriptlib_option_setpermission_fileaccess_all(contextid, "rawi"); // read append write information

    gnscriptlib_context_setintvariable(contextid, "radius", 8);
    gnscriptlib_context_setdoublevariable(contextid, "resultarea", 0.0);
    gnscriptlib_context_setconstant(contextid, "mypi", "3.1415926");


    // module strings uses only strings for all operations including function parameters and results, it requires license with feature mod_strings

    char resultstring[1024];
    // in module strings you must specify maximum size for a result string, in module data types you get a data id for result and can ask the size before getting string data

    if (!gnscriptlib_evaluate_strings(contextid, "pi * sqr(radius)", resultstring, sizeof resultstring, errmsgstring, sizeof errmsgstring))
    {
        std::cerr << errmsgstring<<'\n';
        return 1;
    }
    std::cout << resultstring<< '\n';


    const char* testscriptfile = "test.gnscript";
    FILE* f;
    fopen_s(&f, testscriptfile, "w");
    if (!f)
    {
        std::cerr << "creating file "<< testscriptfile << " failed\n";
        return 1;
    }
    fputs("var area = mypi;\n", f);
    fputs("for (var i = 1; i <=2; i++)\n", f);
    fputs("area *= radius;\n", f);
    fputs("return area;\n", f);
    fclose(f);
    if (!gnscriptlib_execute_strings(contextid, testscriptfile, resultstring, sizeof resultstring, errmsgstring, sizeof errmsgstring))
    {
        std::cerr << errmsgstring<<'\n';
        return 1;
    }
    std::cout << resultstring<< '\n';

    gnscriptlib_context_definefunction_strings(contextid, "calccirclearea(radius)", errmsgstring, sizeof errmsgstring);

    if (!gnscriptlib_evaluate_strings(contextid, "calccirclearea(radius)", resultstring, sizeof resultstring, errmsgstring, sizeof errmsgstring))
    {
        std::cerr << errmsgstring<<'\n';
        return 1;
    }
    std::cout << resultstring<< '\n';

    // module data type can additionally create and access vectors and structures and strings by data ids and function calls that use data ids in parameters and result. It requires license with feature mod_datatypes

    unsigned long vectorid = gnscriptlib_data_createvector(contextid);
    gnscriptlib_data_vector_pushelement(contextid, vectorid, gnscriptlib_data_createlong(contextid, 1));
    gnscriptlib_data_vector_pushelement(contextid, vectorid, gnscriptlib_data_createlong(contextid, 3));
    gnscriptlib_data_vector_pushelement(contextid, vectorid, gnscriptlib_data_createstring(contextid, "four"));
    gnscriptlib_data_vector_pushelement(contextid, vectorid, gnscriptlib_data_createlong(contextid, 5));
    if (!gnscriptlib_context_setvariable(contextid, "myvector", vectorid))
    {
        std::cerr << errmsgstring << '\n';
        return 1;
    }
    if (!gnscriptlib_data_tostring(contextid, vectorid, resultstring, sizeof resultstring))
    {
        std::cerr << "tostring failed" << '\n';
        return 1;
    }
    std::cout << resultstring<< '\n';

    if (!gnscriptlib_evaluate_strings(contextid, "join('=', myvector)", resultstring, sizeof resultstring, errmsgstring, sizeof errmsgstring))
    {
        std::cerr << errmsgstring << '\n';
        return 1;
    }
    std::cout << resultstring<< '\n';
    if (!gnscriptlib_evaluate_strings(contextid, "myvector[1]+myvector[3]", resultstring, sizeof resultstring, errmsgstring, sizeof errmsgstring))
    {
        std::cerr << errmsgstring << '\n';
        return 1;
    }
    std::cout << resultstring<< '\n';

    long structureid = gnscriptlib_data_createstructure(contextid);
    gnscriptlib_data_structure_setmember(contextid, structureid, "name", gnscriptlib_data_createstring(contextid, "guenter"));
    unsigned long substructureid;
    if (!gnscriptlib_evaluate_dataids(contextid, "return { age: 30; title: 'Dipl.-Ing'; };", &substructureid, errmsgstring, sizeof errmsgstring))
    {
        std::cerr << errmsgstring << '\n';
        return 1;
    }
    gnscriptlib_data_structure_setmember(contextid, structureid, "info", substructureid);
//    gnscriptlib_data_structure_setmember(contextid, structureid, "age", gnscriptlib_data_createlong(contextid, 30));
    //gnscriptlib_data_structure_setmember(contextid, structureid, "title", gnscriptlib_data_createstring(contextid, "dipling"));
    if (!gnscriptlib_data_debug(contextid, structureid, resultstring, sizeof resultstring))
    {
        std::cerr << "structure tostring failed" << '\n';
        return 1;
    }
    std::cout << resultstring<< '\n';
    if (!gnscriptlib_context_setvariable(contextid, "mystructure", structureid))
    {
        std::cerr << errmsgstring << '\n';
        return 1;
    }
    if (!gnscriptlib_evaluate_strings(contextid, "mystructure", resultstring, sizeof resultstring, errmsgstring, sizeof errmsgstring))
    {
        std::cerr << errmsgstring << '\n';
        return 1;
    }
    std::cout << resultstring<< '\n';

    unsigned long resultdataid;
    if (!gnscriptlib_evaluate_dataids(contextid, "mystructure.name + '-' + mystructure.info.age", &resultdataid, errmsgstring, sizeof errmsgstring))
    {
        std::cerr << errmsgstring << '\n';
        return 1;
    }
    gnscriptlib_data_tostring(contextid, resultdataid, resultstring, sizeof resultstring);
    std::cout << resultstring<< '\n';

    gnscriptlib_context_definefunction_dataids(contextid, "calccirclearea2(radius, &area)", errmsgstring, sizeof errmsgstring);
    if (!gnscriptlib_evaluate_strings(contextid, "'before calccirclearea2: radius='+radius+ ' resultarea='+resultarea", resultstring, sizeof resultstring, errmsgstring, sizeof errmsgstring))
    {
        std::cerr << errmsgstring<<'\n';
        return 1;
    }
    std::cout << resultstring<< '\n';

    if (!gnscriptlib_evaluate_dataids(contextid, "calccirclearea2(radius, resultarea)", &resultdataid, errmsgstring, sizeof errmsgstring))
    {
        std::cerr << errmsgstring<<'\n';
        return 1;
    }
    gnscriptlib_data_tostring(contextid, resultdataid, resultstring, sizeof resultstring);
    std::cout << "calccirclearea2 returned: " << resultstring<< '\n';

    if (!gnscriptlib_evaluate_strings(contextid, "'after calccirclearea2: radius='+radius+ ' resultarea='+resultarea", resultstring, sizeof resultstring, errmsgstring, sizeof errmsgstring))
    {
        std::cerr << errmsgstring << '\n';
        return 1;
    }
    std::cout << resultstring<< '\n';

    const char* orig = "orig";
    unsigned long origid = gnscriptlib_data_createstring(contextid, orig);
    unsigned long dupid = gnscriptlib_data_duplicate(contextid, origid);
    gnscriptlib_data_tostring(contextid, dupid, resultstring, sizeof resultstring);
    std::cout << "origid=" << origid << " dupid=" << dupid << " orig=" << orig << " dup=" << resultstring << '\n';



    /*
    // module livedata is not yet available it allows to connect your internal data with virtual strings, vectors, structures in script without copying the data
    // example: you define a reference to a picture in your program  the picture is virtually a structure of some picture information and one structure member could be pixels that allows to access color pixels by virtual vectors myimage.pixels[y][x]
    
    usergnscript_data_reference* ref = create_usergnscript_data_reference((unsigned long)(&(song->songname_)), 0, 0); // string
    unsigned long songnamedataid = gnscriptlib_livedata_create(contextid, (unsigned long)ref, DATATYPE_STRING, "SONGNAME", errmsgstring, sizeof errmsgstring);
    gnscriptlib_context_setvariable(contextid, "songname", songnamedataid);

    if (!gnscriptlib_evaluate_strings(contextid, "var copysongname = songname; songname = 'new song'; return copysongname;", resultstring, sizeof resultstring, errmsgstring, sizeof errmsgstring))
    {
        std::cerr << errmsgstring<<'\n';
        return 1;
    }
    std::cout << resultstring<< '\n';
    if (!gnscriptlib_data_tostring(contextid, songnamedataid, resultstring, sizeof resultstring))
    {
        return 1;
    }
    std::cout << resultstring<< '\n';
    */
/*    
    usergnscript_data_reference* songref = create_usergnscript_data_reference(song, 0, 0); // structure SONG
    unsigned long songlivedataid = gnscriptlib_livedata_create(contextid, songref, DATATYPE_STRUCTURE, "SONG", errmsgstring, sizeof errmsgstring);
    gnscriptlib_context_setvariable(contextid, "song", songlivedataid);

    if (!gnscriptlib_evaluate_strings(contextid, "song.songname + ' ' + song.format;", resultstring, sizeof resultstring, errmsgstring, sizeof errmsgstring))
    {
        std::cerr << errmsgstring<<'\n';
        return 1;
    }
    std::cout << resultstring<< '\n';

    if (!gnscriptlib_data_tostring(contextid, songlivedataid, resultstring, sizeof resultstring))
    {
        return 1;
    }
    std::cout << resultstring<< '\n';
*/    

    gnscriptlib_freecontext(contextid);

#ifdef GNUNITTEST
    if (!gnscriptlib_unittests::starttests(USERLICENSEDATA, 0))
    {
          return 1;
    }
#endif
    return 0;
}

